/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_svs.c,v 1.10 2002/12/12 05:31:54 jeremy Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#include "s_user.h"  /* send_umode_out() */
#include "channel_mode.h" /* MODE_ADD & MOD_DEL */


/* Declare the void's initially up here, as modules dont have an
 * include file, we will normally have client_p, source_p, parc
 * and parv[] where:
 *
 * client_p == client issuing command
 * source_p == where the command came from
 * parc     == the number of parameters
 * parv     == an array of the parameters
 */
static void ms_svshost(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void mo_svshost(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void ms_svsnick(struct Client *client_p, struct Client *source_p,
		    int parc, char *parv[]);
static void mo_svsnick(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void mo_svsmode(struct Client *client_p, struct Client *target_p,
                    int parc, char *parv[]);

static int valid_hostname(const char* hostname);
static int clean_nick_name(char *nick);
static void remove_host(struct Client *source_p, char *nick);
static void change_host(struct Client *source_p, char *, char *);

/* CHGHOST/SVSHOST/SETHOST */
struct Message svshost_msgtab = {
  "SVSHOST", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_ignore, m_ignore, ms_svshost, m_ignore}
};
struct Message chghost_msgtab = {
  "CHGHOST", 0, 0, 2, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svshost}
};
struct Message sethost_msgtab = {
  "SETHOST", 0, 0, 2, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svshost}
};

/* CHGNICK/SVSNICK/SETNICK */
struct Message svsnick_msgtab = {
  "SVSNICK", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_ignore, ms_svsnick, m_ignore}
};
struct Message chgnick_msgtab = {
  "CHGNICK", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svsnick}
};
struct Message setnick_msgtab = {
  "SETNICK", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svsnick}
};

/* SVSMODE/CHGMODE/SETMODE */
struct Message svsmode_msgtab = {
  "SVSMODE", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svsmode}
};
struct Message chgmode_msgtab = {
  "CHGMODE", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svsmode}
};
struct Message setmode_msgtab = {
  "SETMODE", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svsmode}
};


#ifndef STATIC_MODULES
/* Here we tell it what to do when the module is loaded */
void
_modinit(void)
{
  /* This will add the commands in test_msgtab (which is above) */
  /* SVSHOST/CHGHOST/SETHOST */
  mod_add_cmd(&svshost_msgtab);
  mod_add_cmd(&chghost_msgtab);
  mod_add_cmd(&sethost_msgtab);
  /* SVSNICK/CHGNICK/SETNICK */
  mod_add_cmd(&svsnick_msgtab);
  mod_add_cmd(&chgnick_msgtab);
  mod_add_cmd(&setnick_msgtab);
  /* SVSMODE/CHGMODE/SETMODE */
  mod_add_cmd(&svsmode_msgtab);
  mod_add_cmd(&chgmode_msgtab);
  mod_add_cmd(&setmode_msgtab);
}

/* here we tell it what to do when the module is unloaded */
void
_moddeinit(void)
{
  /* This will remove the commands in test_msgtab (which is above) */
  /* SVSHOST/CHGHOST/SETHOST */
  mod_del_cmd(&svshost_msgtab);
  mod_del_cmd(&chghost_msgtab);
  mod_del_cmd(&sethost_msgtab);
  /* SVSNICK/CHGNICK/SETNICK */
  mod_del_cmd(&svsnick_msgtab);
  mod_del_cmd(&chgnick_msgtab);
  mod_del_cmd(&setnick_msgtab);
  /* SVSMODE/CHGMODE/SETMODE */
  mod_del_cmd(&svsmode_msgtab);
  mod_del_cmd(&chgmode_msgtab);
  mod_del_cmd(&setmode_msgtab);
}

/* When we last modified the file (shown in /modlist), this is usually:
 */
const char *_version = "$Revision: 1.10 $";
#endif

/*
 * mo_svshost
 *      parv[0] = sender prefix
 *      parv[1] = target nickname
 *      parv[2] = new hostname
 */
static void mo_svshost(struct Client *client_p, struct Client *source_p,
                   int parc, char *parv[])
{
  struct Client *target_p;
  char *nick, hostname[HOSTLEN+1];
  int remove = 0;

  nick = parv[1];

  if(parc > 2)
    strcpy(hostname, parv[2]);

  if(!isalpha(nick[0]))
    if(nick[0] == '-')
      remove = 1;

  if(remove > 0)
    {
      nick++;
      remove_host(source_p, nick);
    }
  else
    {
      if (parc != 3)
        {
          sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, source_p->name, "CHGHOST");
          return;
        }
      change_host(source_p, nick, (hostname)? hostname : NULL);
    }

  return;
}

static void remove_host(struct Client *source_p, char *nick)
{
  struct Client *target_p;

  if ((target_p = find_client(nick)) == NULL)
    {
      sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, target_p->name);
      return;
    }

  strcpy(target_p->host, target_p->realhost);

  sendto_one(source_p, "Reset %s's Host", target_p->name);
  return;
}

static void change_host(struct Client *source_p, char *nick, char *host)
{
  struct Client *target_p;
  char oldhost[HOSTLEN+1];
  int setflags;

  if ((target_p = find_client(nick)) == NULL)
    {
      sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, nick);
      return;
    }

  /* check at this point for a valid hostname, and if its null
     which it shouldn't be at this point */
  if (!valid_hostname(host) || (host == NULL))
    {
      sendto_one(source_p, "%s :Invalid Hostname", host);
      return;
    }

  if(!strchr(host, '.') && !strchr(host, ':'))
    {
      sendto_one(source_p, "Hostname's must contain one . or :");
      return;
    }

  /* Preserve the old hostname until we are finished */
  strcpy(oldhost, target_p->host);

  /* Prevent any unnecessary bursts */
  if(!strcmp(oldhost, host))
    {
      sendto_one(source_p, "%s :Hostname Identical", host);
      return;
    }

  if(MyClient(target_p) && IsMasked(target_p))
    {
      setflags = target_p->umodes;
      target_p->umodes &= ~UMODE_MASKED;
      send_umode_out(target_p, target_p, setflags);
    }

  strcpy(target_p->host, host);
  sendto_realops_flags(UMODE_ALL, L_ALL, "CHGHOST was used on %s (%s -> %s)", nick, oldhost, host);
  ilog(L_NOTICE, "%s changed %s's hostname (%s -> %s)", source_p->name, nick, oldhost, host);
  sendto_server(NULL, NULL, CAP_SVSHOST, NOCAPS, ":%s SVSHOST %s %s %s", source_p->name, target_p->name, oldhost, host);

  sendto_one(source_p, "Changed %s's Host (%s -> %s)", nick, oldhost, host);
  return;
}

/*
 * ms_svshost
 *      parv[0] = sender prefix
 *      parv[1] = target nickname
 *      parv[2] = new hostname
 */
static void ms_svshost(struct Client *client_p, struct Client *source_p,
                   int parc, char *parv[])
{
  struct Client *target_p;
  char *nick;
  int setflags;
  nick = parv[1];

  if (parc != 4)
    return;

  sendto_server(client_p, NULL, CAP_SVSHOST, NOCAPS, ":%s SVSHOST %s %s %s", parv[0], parv[1], parv[2], parv[3]);

  target_p = find_client(nick);  
  strcpy(target_p->host, parv[3]);

  if(!IsServer(client_p))
    {
      sendto_realops_flags(UMODE_ALL, L_ALL, "CHGHOST was used on %s (%s -> %s)", nick, parv[2], parv[3]);
      ilog(L_NOTICE, "%s changed %s's hostname (%s -> %s)", parv[0], nick, parv[2], parv[3]);
    }
  else
    {
      /* Changing the host on a masked user?
         if so pull +X */
      if(MyClient(target_p) && IsMasked(target_p))
        {
          setflags = target_p->umodes;
          target_p->umodes &= ~UMODE_MASKED;
          send_umode_out(target_p, target_p, setflags);
        }
      ilog(L_NOTICE, "Caught SVSHOST from server [%s], Possibly a change of UMODE_MASKED", parv[0]);
    }

  return;
}

/*
 * mo_svsnick
 *    parv[0] = sender prefix
 *    parv[1] = target nickname
 *    parv[2] = new nickname
 */
static void mo_svsnick(struct Client *client_p, struct Client *source_p,
                   int parc, char *parv[])
{
  struct Client *target_p;
  struct Client *myuser_p;
  char *nick, newnick[NICKLEN+1];

  nick = parv[1];
  strcpy(newnick, parv[2]);

  if ((target_p = find_client(nick)) != NULL)
    {
      if (!clean_nick_name(newnick))
      {
        sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME), me.name, source_p->name, newnick);
        return;
      }
  
      if (find_nick_resv(newnick))
      {
        sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE), me.name, source_p->name, newnick);
        return;
      }

      if ((myuser_p = find_client(newnick)))
      {
 
        if (myuser_p == target_p)
        {
          if (strcmp(target_p->name, newnick))
          {
            if (MyClient(target_p))
              change_local_nick(target_p, target_p, newnick);
            else
              sendto_server(client_p, NULL, CAP_SVSNICK, NOCAPS, ":%s SVSNICK %s %s", parv[0], target_p->name, newnick);
          }
          else
            return;
        }
  
        else 
        {
          sendto_one(source_p, form_str(ERR_NICKNAMEINUSE), me.name, source_p->name, newnick);
          return;
        }  
      }

      else
      {
        if (MyClient(target_p))
          change_local_nick(target_p, target_p, newnick);
        else
          sendto_server(client_p, NULL, CAP_SVSNICK, NOCAPS, ":%s SVSNICK %s %s", parv[0], target_p->name, newnick);
      }
    }
  else
    {
      sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, parv[0], nick);
      return;
    }

  sendto_realops_flags(UMODE_ALL, L_ALL, "%s Used CHGNICK On %s", source_p->name, nick);
  ilog(L_NOTICE,"%s Used SVSNICK On %s To Change Their Nick To %s", get_oper_name(source_p), nick, newnick);

  return; 
} 

static void ms_svsnick(struct Client *client_p, struct Client *source_p,
			int parc, char *parv[])
{
  struct Client *target_p;
  char *nick, newnick[NICKLEN+1];

  if (parc != 3)
    return;

  nick = parv[1];
  strcpy(newnick, parv[2]);

  target_p = find_client(nick);

  if (!MyClient(target_p))
    sendto_server(client_p, NULL, CAP_SVSNICK, NOCAPS, ":%s SVSNICK %s %s", parv[0], target_p->name, newnick);
  else
    change_local_nick(target_p, target_p, newnick);

  sendto_realops_flags(UMODE_ALL, L_ALL, "%s Used CHGNICK On %s", parv[0], nick);
  ilog(L_NOTICE, "%s Used SVSNICK On %s To Change Their Nick To %s", parv[0], nick, newnick);

  return;
}

/*
 * mo_svsmode
 *    parv[0] = sender prefix
 *    parv[1] = target nickname
 *    parv[2] = modes
 */

static void mo_svsmode(struct Client *client_p, struct Client *source_p,
			int parc, char *parv[])
{
  struct Client *target_p;
  char *nick;
  char buf[BUFSIZE];

  int flag, i;
  char **p, *m;
  int what, setflags;

  nick = parv[1];

  what = MODE_ADD;

  if((target_p = find_client(nick)) == NULL)
    {
      sendto_one(source_p, "%s :No such nick", nick);
      return;
    }

  setflags = target_p->umodes;

  for(p = &parv[2]; p && *p; p++)
    for(m = *p; *m; m++)
      switch(*m)
        {
          case '+' :
            what = MODE_ADD;
            break;
          case '-' :
            what = MODE_DEL;
            break;
          case 'o' :
            break;

          case ' ' :
          case '\n' :
          case '\r' :
          case '\t' :
            break;

          default :
            if( (flag = user_modes_from_c_to_bitmask[(unsigned char)*m]))
              {
                if((ConfigFileEntry.oper_only_umodes & flag) && (!IsOper(target_p)))
                  ;
                else
                  {
                    if (what == MODE_ADD)
                      target_p->umodes |= flag;
                    else
                      target_p->umodes &= ~flag;
                  }
              }
            break;
        }
  send_umode_out(target_p, target_p, setflags);
  return;
}

/* ms_svsmode
 *    parv[0] = sender prefix
 *    parv[1] = target nickname
 *    parv[2] = modes
 */
/*
static void ms_svsmode(struct Client *client_p, struct Client *source_p, 
                        int parc, char *parv[]) 
{
  return;
}*/

/*
 * valid_hostname - check hostname for validity
 *   
 * Inputs       - pointer to user
 * Output       - YES if valid, NO if not  
 * Side effects - NONE
 *   
 * NOTE: this doesn't allow a hostname to begin with a dot and
 * will not allow more dots than chars.   
 */
static int
valid_hostname(const char* hostname)
{
  const char* p     = hostname;
      
  assert(NULL != p);

  if(hostname == NULL)
    return NO;
      
  if ('.' == *p)
    return NO;
      
  while (*p)
    {
      if (!IsHostChar(*p))
        return NO;
      p++;
    }
  return (YES);
}

/* clean_nick_name()
 *
 * input        - nickname
 * output       - none
 * side effects - walks through the nickname, returning 0 if erroneous
 */
static int clean_nick_name(char *nick)
{
  assert(nick);
  if(nick == NULL)
    return 0;

  /* nicks cant start with a digit or -, and must have a length */
  if (*nick == '-' || IsDigit(*nick) || !*nick)
    return 0;

  for(; *nick; nick++)
  {
    if(!IsNickChar(*nick))
      return 0;
  }
  
  return 1;
}

/* END OF EXAMPLE MODULE */

